@@ -35,7 +35,6 @@
 #include <e2k-utils.h>
 #include <e2k-global-catalog-ldap.h>
 #include <exchange-account.h>
-#include "exchange-component.h"
 
 #define d(x)
 
@@ -50,9 +49,14 @@
 #include <libedata-book/e-data-book.h>
 #include <libedata-book/e-data-book-view.h>
 #include "libedata-book/e-book-backend-summary.h"
+#include "tools/exchange-share-config-listener.h"
 #include "e-book-backend-gal.h"
 #include <libical/ical.h>
 
+#ifdef _WIN32
+#include <winber.h>
+#endif
+
 #ifndef LDAP_CONTROL_PAGEDRESULTS
 #ifdef ENABLE_CACHE
 #undef ENABLE_CACHE
@@ -216,8 +220,6 @@ struct prop_info {
 #undef GROUP_PROP
 };
 
-static gint num_prop_infos = sizeof(prop_info) / sizeof(prop_info[0]);
-
 static gboolean
 can_browse (EBookBackend *backend)
 {
@@ -227,6 +229,15 @@ can_browse (EBookBackend *backend)
 		strcmp (e_source_get_property (e_book_backend_get_source (backend), "can-browse"), "1") == 0;
 }
 
+static gboolean
+can_expand_groups (EBookBackend *backend)
+{
+	return backend &&
+		e_book_backend_get_source (backend) &&
+		e_source_get_property (e_book_backend_get_source (backend), "expand-groups") &&
+		strcmp (e_source_get_property (e_book_backend_get_source (backend), "expand-groups"), "1") == 0;
+}
+
 static void
 book_view_notify_status (EDataBookView *view, const gchar *status)
 {
@@ -271,7 +282,7 @@ gal_connect (EBookBackendGAL *bl)
 	blpriv->gc = NULL;
 	blpriv->connected = FALSE;
 
-	blpriv->account = exchange_component_get_account_for_uri (global_exchange_component, blpriv->gal_uri);
+	blpriv->account = exchange_share_config_listener_get_account_for_uri (NULL, blpriv->gal_uri);
 	if (!blpriv->account)
 		return GNOME_Evolution_Addressbook_RepositoryOffline;
 	blpriv->gc = exchange_account_get_global_catalog (blpriv->account);
@@ -920,9 +931,9 @@ func_and(ESExp *f, gint argc, ESExpResult **argv, gpointer data)
 	/* Check for short circuit */
 	for (i = 0; i < argc; i++) {
 		if (argv[i]->type == ESEXP_RES_BOOL &&
-		    argv[i]->value.bool == FALSE) {
+		    argv[i]->value.boolean == FALSE) {
 			r = e_sexp_result_new(f, ESEXP_RES_BOOL);
-			r->value.bool = FALSE;
+			r->value.boolean = FALSE;
 			return r;
 		} else if (argv[i]->type == ESEXP_RES_UNDEFINED)
 			return e_sexp_result_new(f, ESEXP_RES_UNDEFINED);
@@ -953,9 +964,9 @@ func_or(ESExp *f, gint argc, ESExpResult **argv, gpointer data)
 	/* Check for short circuit */
 	for (i = 0; i < argc; i++) {
 		if (argv[i]->type == ESEXP_RES_BOOL &&
-		    argv[i]->value.bool == TRUE) {
+		    argv[i]->value.boolean == TRUE) {
 			r = e_sexp_result_new(f, ESEXP_RES_BOOL);
-			r->value.bool = TRUE;
+			r->value.boolean = TRUE;
 			return r;
 		} else if (argv[i]->type == ESEXP_RES_UNDEFINED)
 			return e_sexp_result_new(f, ESEXP_RES_UNDEFINED);
@@ -992,7 +1003,7 @@ func_not(ESExp *f, gint argc, ESExpResult **argv, gpointer data)
 						   argv[0]->value.string);
 	} else {
 		r = e_sexp_result_new(f, ESEXP_RES_BOOL);
-		r->value.bool = !argv[0]->value.bool;
+		r->value.boolean = !argv[0]->value.boolean;
 	}
 
 	return r;
@@ -1006,7 +1017,7 @@ query_prop_to_ldap (const gchar *query_prop)
 	if (!strcmp (query_prop, "email"))
 		query_prop = "email_1";
 
-	for (i = 0; i < num_prop_infos; i ++)
+	for (i = 0; i < G_N_ELEMENTS (prop_info); i ++)
 		if (!strcmp (query_prop, e_contact_field_name (prop_info[i].field_id)))
 			return prop_info[i].ldap_attr;
 
@@ -1036,7 +1047,7 @@ func_contains(ESExp *f, gint argc, ESExpResult **argv, gpointer data)
 		 */
 		if (strlen(str) == 0) {
 			r = e_sexp_result_new(f, ESEXP_RES_BOOL);
-			r->value.bool = FALSE;
+			r->value.boolean = FALSE;
 		} else {
 			r = e_sexp_result_new(f, ESEXP_RES_STRING);
 			r->value.string = g_strdup_printf ("(mailNickname=%s)", str);
@@ -1049,7 +1060,7 @@ func_contains(ESExp *f, gint argc, ESExpResult **argv, gpointer data)
 	if (!ldap_attr) {
 		/* Attribute doesn't exist, so it can't possibly match */
 		r = e_sexp_result_new(f, ESEXP_RES_BOOL);
-		r->value.bool = FALSE;
+		r->value.boolean = FALSE;
 		return r;
 	}
 
@@ -1110,7 +1121,7 @@ func_is_or_begins_with(ESExp *f, gint argc, ESExpResult **argv, gboolean exact)
 
 		/* Property doesn't exist, so it can't ever match */
 		r = e_sexp_result_new(f, ESEXP_RES_BOOL);
-		r->value.bool = FALSE;
+		r->value.boolean = FALSE;
 		return r;
 	}
 
@@ -1190,7 +1201,7 @@ build_query (EBookBackendGAL *bl, const gchar *query, const gchar *ldap_filter,
 
 	sexp = e_sexp_new();
 
-	for (i=0;i<sizeof(symbols)/sizeof(symbols[0]);i++) {
+	for (i = 0; i < G_N_ELEMENTS (symbols); i++) {
 		e_sexp_add_function(sexp, 0, (gchar *) symbols[i].name,
 				    symbols[i].func, NULL);
 	}
@@ -1382,7 +1393,7 @@ build_contact_from_entry (EBookBackendGAL *bl, LDAPMessage *e, GList **existing_
 	EContact *contact = e_contact_new ();
 	gchar *dn;
 	gchar *attr;
-	BerElement *ber = NULL, *tber = NULL;
+	BerElement *ber = NULL;
 	gboolean is_group = FALSE;
 
 	g_mutex_lock (bl->priv->ldap_lock);
@@ -1392,21 +1403,25 @@ build_contact_from_entry (EBookBackendGAL *bl, LDAPMessage *e, GList **existing_
 	e_contact_set (contact, E_CONTACT_UID, dn);
 	ldap_memfree (dn);
 
-	g_mutex_lock (bl->priv->ldap_lock);
-	attr = ldap_first_attribute (bl->priv->ldap, e, &tber);
-	while (attr) {
-		if (!strcmp(attr, "member")) {
-			d(printf("It is a DL\n"));
-			is_group = TRUE;
+	if (can_expand_groups (E_BOOK_BACKEND (bl))) {
+		BerElement *tber = NULL;
+
+		g_mutex_lock (bl->priv->ldap_lock);
+		attr = ldap_first_attribute (bl->priv->ldap, e, &tber);
+		while (attr) {
+			if (!strcmp(attr, "member")) {
+				d(printf("It is a DL\n"));
+				is_group = TRUE;
+				ldap_memfree (attr);
+				break;
+			}
 			ldap_memfree (attr);
-			break;
+			attr = ldap_next_attribute (bl->priv->ldap, e, tber);
 		}
-		ldap_memfree (attr);
-		attr = ldap_next_attribute (bl->priv->ldap, e, tber);
+		if (tber)
+			ber_free (tber, 0);
+		g_mutex_unlock (bl->priv->ldap_lock);
 	}
-	if (tber)
-		ber_free (tber, 0);
-	g_mutex_unlock (bl->priv->ldap_lock);
 
 	g_mutex_lock (bl->priv->ldap_lock);
 	attr = ldap_first_attribute (bl->priv->ldap, e, &ber);
@@ -1433,7 +1448,7 @@ build_contact_from_entry (EBookBackendGAL *bl, LDAPMessage *e, GList **existing_
 			ldap_value_free (values);
 		}
 		else {
-			for (i = 0; i < num_prop_infos; i ++)
+			for (i = 0; i < G_N_ELEMENTS (prop_info); i ++)
 				if (!g_ascii_strcasecmp (attr, prop_info[i].ldap_attr)) {
 					info = &prop_info[i];
 					break;
@@ -1460,7 +1475,7 @@ build_contact_from_entry (EBookBackendGAL *bl, LDAPMessage *e, GList **existing_
 							   which calls g_object_set to set the property */
 							info->populate_contact_func(contact, values, bl, NULL);
 						}
-						else if (info->prop_type & PROP_TYPE_GROUP) {
+						else if (is_group && (info->prop_type & PROP_TYPE_GROUP)) {
 							gchar *grpattrs[3];
 							gint i, view_limit = -1, ldap_error = LDAP_SUCCESS, count;
 							EDataBookView *book_view;
@@ -1711,7 +1726,7 @@ ldap_search_dtor (LDAPOp *op)
 	d(printf ("ldap_search_dtor: Setting null inplace of %p in view %p\n", op, search_op->view));
 	g_object_set_data (G_OBJECT (search_op->view), "EBookBackendGAL.BookView::search_op", NULL);
 
-	bonobo_object_unref (search_op->view);
+	e_data_book_view_unref (search_op->view);
 
 	if (!search_op->aborted)
 		g_free (search_op);
@@ -1923,7 +1938,7 @@ start_book_view (EBookBackend  *backend,
 				op->view = view;
 				op->aborted = FALSE;
 
-				bonobo_object_ref (view);
+				e_data_book_view_ref (view);
 
 				ldap_op_add ((LDAPOp*)op, E_BOOK_BACKEND (bl), NULL, view,
 					     0, search_msgid,
@@ -1982,28 +1997,28 @@ tool_server_controls( LDAP *ld, LDAPControl *extra_c, gint count )
 	LDAPControl c[3], **ctrls;
 
 	ctrls = (LDAPControl**) malloc(sizeof(c) + (count+1)*sizeof(LDAPControl*));
-	if ( ctrls == NULL ) {
+	if (ctrls == NULL) {
 		fprintf( stderr, "No memory\n" );
 		exit( EXIT_FAILURE );
 	}
 
-	while ( count-- ) {
+	while (count--) {
 		ctrls[i++] = extra_c++;
 	}
 	ctrls[i] = NULL;
 
 	err = ldap_set_option( ld, LDAP_OPT_SERVER_CONTROLS, ctrls );
 
-	if ( err != LDAP_SUCCESS ) {
+	if (err != LDAP_SUCCESS) {
 		for ( j = 0; j < i; j++ ) {
-			if ( ctrls[j]->ldctl_iscritical ) crit = 1;
+			if (ctrls[j]->ldctl_iscritical) crit = 1;
 		}
 		fprintf( stderr, "Could not set %scontrols\n",
 			crit ? "critical " : "" );
 	}
 
 	free( ctrls );
-	if ( crit ) {
+	if (crit) {
 		exit( EXIT_FAILURE );
 	}
 }
@@ -2045,16 +2060,16 @@ parse_page_control(
 	rc = ldap_parse_result( ld, result,
 		&err, NULL, NULL, NULL, &ctrl, 0 );
 
-	if ( rc != LDAP_SUCCESS ) {
+	if (rc != LDAP_SUCCESS) {
 		ldap_perror(ld, "ldap_parse_result");
 		exit( EXIT_FAILURE );
 	}
 
-	if ( err != LDAP_SUCCESS ) {
+	if (err != LDAP_SUCCESS) {
 		fprintf( stderr, "Error: %s (%d)\n", ldap_err2string(err), err );
 	}
 
-	if ( ctrl ) {
+	if (ctrl) {
 		/* Parse the control value
 		 * searchResult ::= SEQUENCE {
 		 *		size	INTEGER (0..maxInt),
@@ -2064,7 +2079,7 @@ parse_page_control(
 		 */
 		ctrlp = *ctrl;
 		ber = ber_init( &ctrlp->ldctl_value );
-		if ( ber == NULL ) {
+		if (ber == NULL) {
 			fprintf( stderr, "Internal error.\n");
 			return EXIT_FAILURE;
 		}
@@ -2073,13 +2088,13 @@ parse_page_control(
 		ber_dupbv( cookie, &servercookie );
 		(void) ber_free( ber, 1 );
 
-		if ( tag == LBER_ERROR ) {
+		if (tag == LBER_ERROR) {
 			fprintf( stderr,
 				"Paged results response control could not be decoded.\n");
 			return EXIT_FAILURE;
 		}
 
-		if ( entriesLeft < 0 ) {
+		if (entriesLeft < 0) {
 			fprintf( stderr,
 				"Invalid entries estimate in paged results response.\n");
 			return EXIT_FAILURE;
@@ -2129,7 +2144,7 @@ static gint dosearch(
 		sctrls, cctrls, timeout, size /*LDAP_NO_LIMIT*/, &msgid );
 	g_mutex_unlock (bl->priv->ldap_lock);
 
-	if ( rc != LDAP_SUCCESS ) {
+	if (rc != LDAP_SUCCESS) {
 		return( rc );
 	}
 
@@ -2147,7 +2162,7 @@ static gint dosearch(
 			EContact *contact;
 			const gchar *uid;
 
-			switch ( ldap_msgtype( msg ) ) {
+			switch (ldap_msgtype( msg )) {
 			case LDAP_RES_SEARCH_ENTRY:
 				count ++;
 				g_mutex_unlock (bl->priv->ldap_lock);
@@ -2172,7 +2187,7 @@ static gint dosearch(
 				break;
 
 			case LDAP_RES_SEARCH_RESULT:
-				if ( pageSize != 0 ) {
+				if (pageSize != 0) {
 					/* we hold the lock already */
 					rc = parse_page_control (bl->priv->ldap, msg, &cookie);
 				}
@@ -2187,7 +2202,7 @@ static gint dosearch(
 	}
 	g_mutex_unlock (bl->priv->ldap_lock);
 
-	if ( rc == -1 ) {
+	if (rc == -1) {
 		g_mutex_lock (bl->priv->ldap_lock);
 		ldap_perror (bl->priv->ldap, "ldap_result");
 		g_mutex_unlock (bl->priv->ldap_lock);
@@ -2228,14 +2243,28 @@ getNextPage:
 	/*start iteration*/
 
 	i = 0;
-	if ( pagedResults ) {
+	if (pagedResults) {
+#ifdef G_OS_WIN32
+		struct berval **tmpBVPtr = NULL;
+#endif
 		if (( prber = ber_alloc_t(LBER_USE_DER)) == NULL ) {
 			return;
 		}
 		ber_printf( prber, "{iO}", pageSize, &cookie );
-		if ( ber_flatten2( prber, &c[i].ldctl_value, 0 ) == -1 ) {
+#ifdef G_OS_WIN32
+		if (ber_flatten( prber, tmpBVPtr) == -1) {
+			ber_free( prber, 1 );
+			ber_bvfree(*tmpBVPtr);
 			return;
 		}
+		c[i].ldctl_value = **tmpBVPtr;
+		ber_free( prber, 1 );
+		ber_bvfree(*tmpBVPtr);
+#else
+		if (ber_flatten2( prber, &c[i].ldctl_value, 0 ) == -1) {
+			return;
+		}
+#endif
 		d(printf ("Setting parameters		\n"));
 		c[i].ldctl_oid = (gchar *) LDAP_CONTROL_PAGEDRESULTS;
 		c[i].ldctl_iscritical = pagedResults > 1;
@@ -2259,7 +2288,7 @@ getNextPage:
 
 	/* loop to get the next set of entries */
 
-	if ( (pageSize !=0 ) && (morePagedResults != 0)) {
+	if ((pageSize !=0) && (morePagedResults != 0)) {
 		d(printf ("Start next iteration\n"));
 		goto getNextPage;
 	} else {
@@ -2325,7 +2354,7 @@ authenticate_user (EBookBackend *backend,
 	EBookBackendGAL *be = E_BOOK_BACKEND_GAL (backend);
 	EBookBackendGALPrivate *bepriv = be->priv;
 	ExchangeAccountResult result;
-	ExchangeAccount *account;
+	ExchangeAccount *account = NULL;
 	GNOME_Evolution_Addressbook_CallStatus res;
 	GConfClient *gc = gconf_client_get_default();
 	gint interval = gconf_client_get_int (gc, "/apps/evolution/addressbook/gal_cache_interval", NULL);
@@ -2348,7 +2377,7 @@ authenticate_user (EBookBackend *backend,
 
 	case GNOME_Evolution_Addressbook_MODE_REMOTE:
 
-		account = exchange_component_get_account_for_uri (global_exchange_component, NULL);
+		account = exchange_share_config_listener_get_account_for_uri (NULL, bepriv->gal_uri);
 		/* FIXME : Check for failures */
 		if (!exchange_account_get_context (account)) {
 			exchange_account_set_online (account);
@@ -2412,7 +2441,7 @@ ber_flatten2( BerElement *ber, struct berval *bv, gint alloc )
 {
 	struct berval *tmp;
 
-	if ( ber_flatten( ber, &tmp) == -1 ) {
+	if (ber_flatten( ber, &tmp) == -1) {
 		return;
 	}
 	bv->bv_len = tmp->bv_len;
@@ -2451,7 +2480,8 @@ cancel_operation (EBookBackend *backend, EDataBook *book)
 }
 
 static void
-set_mode (EBookBackend *backend, gint mode)
+set_mode (EBookBackend *backend,
+          GNOME_Evolution_Addressbook_BookMode mode)
 {
 	EBookBackendGAL *be = E_BOOK_BACKEND_GAL (backend);
 	EBookBackendGALPrivate *bepriv;
@@ -2544,7 +2574,7 @@ load_source (EBookBackend *backend,
 	gchar **tokens;
 	const gchar *offline;
 	gchar *uri;
-	gchar *book_name;
+	gchar *book_name = NULL;
 	gchar *dirname, *filename;
 	gint i, db_error;
 #if ENABLE_CACHE
@@ -2569,12 +2599,11 @@ load_source (EBookBackend *backend,
 
 	bl->priv->gal_uri = g_strdup (uri);
 	tokens = g_strsplit (uri, ";", 2);
-	g_free (uri);
-	if (tokens[0])
+	if (tokens[0]) {
+		g_free (uri);
 		uri = g_strdup (tokens [0]);
-	book_name = g_strdup (tokens[1]);
-	if (book_name == NULL)
-		return GNOME_Evolution_Addressbook_OtherError;
+		book_name = g_strdup (tokens[1]);
+	}
 	g_strfreev (tokens);
 
 	for (i=0; i< strlen (uri); i++) {
@@ -2595,6 +2624,9 @@ load_source (EBookBackend *backend,
 		e_book_backend_notify_writable (backend, FALSE);
 		e_book_backend_notify_connection_status (backend, FALSE);
 
+		g_free (book_name);
+		g_free (uri);
+
 		return GNOME_Evolution_Addressbook_RepositoryOffline;
 	}
 		d(printf("offlin==============\n"));
@@ -2610,6 +2642,9 @@ load_source (EBookBackend *backend,
 		dirname = g_build_filename (g_get_home_dir(), ".evolution/cache/addressbook", uri, book_name, NULL);
 		filename = g_build_filename (dirname, "cache.db", NULL);
 
+		g_free (book_name);
+		g_free (uri);
+
 		db_error = e_db3_utils_maybe_recover (filename);
 		if (db_error != 0) {
 			g_warning ("db recovery failed with %d", db_error);
@@ -2707,6 +2742,8 @@ load_source (EBookBackend *backend,
 		e_book_backend_db_cache_set_filename (bl->priv->file_db, filename);
 		g_free (filename);
 		g_free (dirname);
+	} else {
+		g_free (book_name);
 		g_free (uri);
 	}
 #endif
@@ -2856,7 +2893,7 @@ class_init (EBookBackendGALClass *klass)
 	backend_class->get_changes                = get_changes;
 	backend_class->authenticate_user          = authenticate_user;
 	backend_class->get_supported_fields       = get_supported_fields;
-	backend_class->set_mode		  = set_mode;
+	backend_class->set_mode                   = set_mode;
 	backend_class->get_required_fields        = get_required_fields;
 	backend_class->get_supported_auth_methods = get_supported_auth_methods;
 	backend_class->cancel_operation           = cancel_operation;
@@ -2865,16 +2902,16 @@ class_init (EBookBackendGALClass *klass)
 
 	/* Set up static data */
 	supported_fields = NULL;
-	for (i = 0; i < num_prop_infos; i++) {
+	for (i = 0; i < G_N_ELEMENTS (prop_info); i++) {
 		supported_fields = g_list_append (supported_fields,
 						  (gchar *)e_contact_field_name (prop_info[i].field_id));
 	}
 	supported_fields = g_list_append (supported_fields, (gpointer) "file_as");
 
-	search_attrs = g_new (const gchar *, num_prop_infos + 1);
-	for (i = 0; i < num_prop_infos; i++)
+	search_attrs = g_new (const gchar *, G_N_ELEMENTS (prop_info) + 1);
+	for (i = 0; i < G_N_ELEMENTS (prop_info); i++)
 		search_attrs[i] = prop_info[i].ldap_attr;
-	search_attrs[num_prop_infos] = NULL;
+	search_attrs[G_N_ELEMENTS (prop_info)] = NULL;
 }
 
 static void
