@@ -27,15 +27,10 @@
 #include <string.h>
 #include <stdlib.h>
 
-#include <camel/camel-db.h>
-#include <camel/camel-file-utils.h>
-#include <camel/camel-offline-store.h>
-#include <camel/camel-string-utils.h>
-
-#include "camel-stub.h"
 #include "camel-exchange-folder.h"
 #include "camel-exchange-journal.h"
 #include "camel-exchange-summary.h"
+#include "camel-exchange-utils.h"
 
 #define CAMEL_EXCHANGE_SUMMARY_VERSION (2)
 
@@ -46,145 +41,63 @@
 
 #define d(x)
 
-static gint header_load (CamelFolderSummary *summary, FILE *in);
-static gint header_save (CamelFolderSummary *summary, FILE *out);
+G_DEFINE_TYPE (CamelExchangeSummary, camel_exchange_summary, CAMEL_TYPE_FOLDER_SUMMARY)
 
-static CamelMessageInfo *message_info_load (CamelFolderSummary *summary,
-					    FILE *in);
-static gint               message_info_save (CamelFolderSummary *summary,
-					    FILE *out,
-					    CamelMessageInfo *info);
-static gint summary_header_from_db (CamelFolderSummary *s, CamelFIRecord *mir);
-static CamelFIRecord * summary_header_to_db (CamelFolderSummary *s, CamelException *ex);
-static CamelMIRecord * message_info_to_db (CamelFolderSummary *s, CamelMessageInfo *info);
-static CamelMessageInfo * message_info_from_db (CamelFolderSummary *s, CamelMIRecord *mir);
-static CamelMessageInfo *message_info_new_from_header  (CamelFolderSummary *summary,
-							struct _camel_header_raw *h);
-
-static void message_info_free (CamelFolderSummary *summary, CamelMessageInfo *info);
-
-static gboolean check_for_trash (CamelFolder *folder);
-static gboolean expunge_mail (CamelFolder *folder, CamelMessageInfo *info);
-
-static gboolean info_set_flags(CamelMessageInfo *info, guint32 flags, guint32 set);
-static gboolean info_set_user_tag(CamelMessageInfo *info, const gchar *name, const gchar *value);
-
-static CamelFolderSummaryClass *parent_class = NULL;
-
-static void
-exchange_summary_class_init (CamelObjectClass *klass)
+static gboolean
+exchange_summary_check_for_trash (CamelFolder *folder)
 {
-	CamelFolderSummaryClass *camel_folder_summary_class =
-		(CamelFolderSummaryClass *) klass;
-
-	parent_class = CAMEL_FOLDER_SUMMARY_CLASS (camel_type_get_global_classfuncs (camel_folder_summary_get_type()));
-
-	camel_folder_summary_class->summary_header_load = header_load;
-	camel_folder_summary_class->summary_header_save = header_save;
-	camel_folder_summary_class->message_info_load = message_info_load;
-	camel_folder_summary_class->message_info_save = message_info_save;
-	camel_folder_summary_class->message_info_new_from_header = message_info_new_from_header;
-	camel_folder_summary_class->message_info_free = message_info_free;
-
-	camel_folder_summary_class->summary_header_to_db = summary_header_to_db;
-	camel_folder_summary_class->summary_header_from_db = summary_header_from_db;
-	camel_folder_summary_class->message_info_to_db = message_info_to_db;
-	camel_folder_summary_class->message_info_from_db = message_info_from_db;
-
-	camel_folder_summary_class->info_set_flags = info_set_flags;
-	camel_folder_summary_class->info_set_user_tag = info_set_user_tag;
-}
-
-static void
-exchange_summary_init (CamelObject *obj, CamelObjectClass *klass)
-{
-	CamelFolderSummary *summary = (CamelFolderSummary *)obj;
-
-	summary->message_info_size = sizeof (CamelExchangeMessageInfo);
-	summary->content_info_size = sizeof (CamelMessageContentInfo);
-}
-
-CamelType
-camel_exchange_summary_get_type (void)
-{
-	static CamelType type = CAMEL_INVALID_TYPE;
-
-	if (type == CAMEL_INVALID_TYPE) {
-		type = camel_type_register(
-			camel_folder_summary_get_type (),
-			"CamelExchangeSummary",
-			sizeof (CamelExchangeSummary),
-			sizeof (CamelExchangeSummaryClass),
-			exchange_summary_class_init,
-			NULL,
-			exchange_summary_init,
-			NULL);
-	}
-
-	return type;
-}
-
-/**
- * camel_exchange_summary_new:
- * @filename: filename to use for the summary
- *
- * Creates a new #CamelExchangeSummary based on @filename.
- *
- * Return value: the summary object.
- **/
-CamelFolderSummary *
-camel_exchange_summary_new (struct _CamelFolder *folder, const gchar *filename)
-{
-	CamelFolderSummary *summary;
+	CamelStore *parent_store;
+	CamelFolder *trash;
 	CamelException lex;
 
 	camel_exception_init (&lex);
-	summary = (CamelFolderSummary *)camel_object_new (CAMEL_EXCHANGE_SUMMARY_TYPE);
-	summary->folder = folder;
-	camel_folder_summary_set_filename (summary, filename);
-	if (camel_folder_summary_load_from_db (summary, &lex) == -1) {
-		g_warning ("Unable to load Exchage summary for folder %s: %s\n", folder->full_name, camel_exception_get_description(&lex));
-		camel_folder_summary_clear_db (summary);
-		camel_folder_summary_touch (summary);
-	}
 
-	camel_exception_clear (&lex);
-	return summary;
+	parent_store = camel_folder_get_parent_store (folder);
+	trash = camel_store_get_trash (parent_store, &lex);
+
+	if (camel_exception_is_set (&lex) || !trash)
+		return FALSE;
+
+	return folder == trash;
 }
 
-static gint
-summary_header_from_db (CamelFolderSummary *s, CamelFIRecord *mir)
+static gboolean
+exchange_summary_expunge_mail (CamelFolder *folder,
+                               CamelMessageInfo *info)
 {
-	CamelExchangeSummary *exchange = (CamelExchangeSummary *) s;
-	gchar *part;
+	GPtrArray *uids = g_ptr_array_new ();
+	CamelStore *parent_store;
+	gchar *uid = g_strdup (info->uid);
+	CamelException lex;
+	const gchar *full_name;
 
-	if (CAMEL_FOLDER_SUMMARY_CLASS (parent_class)->summary_header_from_db (s, mir) == -1)
-		return -1;
+	camel_exception_init (&lex);
 
-	part = mir->bdata;
+	full_name = camel_folder_get_full_name (folder);
+	parent_store = camel_folder_get_parent_store (folder);
 
-	if (part) {
-		EXTRACT_FIRST_DIGIT (exchange->version)
-	}
+	g_ptr_array_add (uids, uid);
 
-	if (part) {
-		EXTRACT_DIGIT (exchange->readonly)
-	}
+	camel_exchange_utils_expunge_uids (
+		CAMEL_SERVICE (parent_store), full_name, uids, &lex);
 
-	if (part) {
-		EXTRACT_DIGIT (exchange->high_article_num)
-	}
+	g_ptr_array_free (uids, TRUE);
 
-	return 0;
+	return camel_exception_is_set (&lex);
 }
 
 static gint
-header_load (CamelFolderSummary *summary, FILE *in)
+exchange_summary_header_load (CamelFolderSummary *summary,
+                              FILE *in)
 {
 	CamelExchangeSummary *exchange = (CamelExchangeSummary *) summary;
+	CamelFolderSummaryClass *folder_summary_class;
 	guint32 version, readonly, high_article_num = 0;
 
-	if (CAMEL_FOLDER_SUMMARY_CLASS (parent_class)->summary_header_load (summary, in) == -1)
+	folder_summary_class = CAMEL_FOLDER_SUMMARY_CLASS (
+		camel_exchange_summary_parent_class);
+
+	if (folder_summary_class->summary_header_load (summary, in) == -1)
 		return -1;
 
 	if (camel_file_util_decode_uint32 (in, &version) == -1)
@@ -208,32 +121,22 @@ header_load (CamelFolderSummary *summary, FILE *in)
 	exchange->high_article_num = high_article_num;
 	exchange->version = version;
 
-
 	d(g_print ("%s:%s: high_article_num = [%d]\n", G_STRLOC, G_STRFUNC, high_article_num));
 
 	return 0;
 }
 
-static CamelFIRecord *
-summary_header_to_db (CamelFolderSummary *s, CamelException *ex)
-{
-	CamelExchangeSummary *exchange = (CamelExchangeSummary *) s;
-	struct _CamelFIRecord *fir;
-
-	fir = CAMEL_FOLDER_SUMMARY_CLASS(parent_class)->summary_header_to_db (s, ex);
-	if (!fir)
-		return NULL;
-	fir->bdata = g_strdup_printf ("%u %u %u", exchange->version, exchange->readonly, exchange->high_article_num);
-
-	return fir;
-}
-
 static gint
-header_save (CamelFolderSummary *summary, FILE *out)
+exchange_summary_header_save (CamelFolderSummary *summary,
+                              FILE *out)
 {
 	CamelExchangeSummary *exchange = (CamelExchangeSummary *) summary;
+	CamelFolderSummaryClass *folder_summary_class;
 
-	if (CAMEL_FOLDER_SUMMARY_CLASS (parent_class)->summary_header_save (summary, out) == -1)
+	folder_summary_class = CAMEL_FOLDER_SUMMARY_CLASS (
+		camel_exchange_summary_parent_class);
+
+	if (folder_summary_class->summary_header_save (summary, out) == -1)
 		return -1;
 
 	if (camel_file_util_encode_uint32 (out, exchange->version) == -1)
@@ -249,32 +152,20 @@ header_save (CamelFolderSummary *summary, FILE *out)
 
 	return 0;
 }
+
 static CamelMessageInfo *
-message_info_from_db (CamelFolderSummary *s, CamelMIRecord *mir)
-{
-	CamelMessageInfo *info;
-	CamelExchangeMessageInfo *einfo;
-
-	info = CAMEL_FOLDER_SUMMARY_CLASS(parent_class)->message_info_from_db (s, mir);
-	if (info) {
-		gchar *part = g_strdup (mir->bdata);
-		gint len;
-		einfo = (CamelExchangeMessageInfo *)info;
-		EXTRACT_FIRST_STRING (einfo->thread_index)
-		EXTRACT_FIRST_STRING (einfo->href)
-	}
-
-	return info;
-}
-
-static CamelMessageInfo *
-message_info_load (CamelFolderSummary *summary, FILE *in)
+exchange_summary_message_info_migrate (CamelFolderSummary *summary,
+                                       FILE *in)
 {
 	CamelMessageInfo *info;
 	CamelExchangeMessageInfo *einfo;
+	CamelFolderSummaryClass *folder_summary_class;
 	gchar *thread_index, *href = NULL;
 
-	info = CAMEL_FOLDER_SUMMARY_CLASS (parent_class)->message_info_load (summary, in);
+	folder_summary_class = CAMEL_FOLDER_SUMMARY_CLASS (
+		camel_exchange_summary_parent_class);
+
+	info = folder_summary_class->message_info_migrate (summary, in);
 	if (info) {
 		einfo = (CamelExchangeMessageInfo *)info;
 
@@ -304,46 +195,19 @@ error:
 	return NULL;
 }
 
-static CamelMIRecord *
-message_info_to_db (CamelFolderSummary *s, CamelMessageInfo *info)
-{
-	CamelExchangeMessageInfo *einfo = (CamelExchangeMessageInfo *)info;
-	struct _CamelMIRecord *mir;
-
-	mir = CAMEL_FOLDER_SUMMARY_CLASS(parent_class)->message_info_to_db (s, info);
-	if (mir)
-		mir->bdata = g_strdup_printf ("%d-%s %d-%s", einfo->thread_index ? (gint)strlen(einfo->thread_index):0 , einfo->thread_index ? einfo->thread_index : "", einfo->href ? (gint)strlen(einfo->href):0, einfo->href ? einfo->href:"");
-
-	return mir;
-}
-
-static gint
-message_info_save (CamelFolderSummary *summary, FILE *out, CamelMessageInfo *info)
-{
-	CamelExchangeMessageInfo *einfo = (CamelExchangeMessageInfo *)info;
-
-	if (CAMEL_FOLDER_SUMMARY_CLASS (parent_class)->message_info_save (summary, out, info) == -1)
-		return -1;
-
-	if (camel_file_util_encode_string (out, einfo->thread_index ? einfo->thread_index : "") == -1)
-		return -1;
-
-	if (camel_file_util_encode_string (out, einfo->href ? einfo->href : "") == -1)
-		return -1;
-
-	d(g_print ("%s:%s: einfo->href = [%s]\n", G_STRLOC, G_STRFUNC, einfo->href));
-
-	return 0;
-}
-
 static CamelMessageInfo *
-message_info_new_from_header (CamelFolderSummary *summary, struct _camel_header_raw *h)
+exchange_summary_message_info_new_from_header (CamelFolderSummary *summary,
+                                               struct _camel_header_raw *h)
 {
 	CamelMessageInfo *info;
 	CamelExchangeMessageInfo *einfo;
+	CamelFolderSummaryClass *folder_summary_class;
 	const gchar *thread_index;
 
-	info = CAMEL_FOLDER_SUMMARY_CLASS (parent_class)->message_info_new_from_header (summary, h);
+	folder_summary_class = CAMEL_FOLDER_SUMMARY_CLASS (
+		camel_exchange_summary_parent_class);
+
+	info = folder_summary_class->message_info_new_from_header (summary, h);
 	if (!info)
 		return info;
 
@@ -356,9 +220,14 @@ message_info_new_from_header (CamelFolderSummary *summary, struct _camel_header_
 }
 
 static void
-message_info_free (CamelFolderSummary *summary, CamelMessageInfo *info)
+exchange_summary_message_info_free (CamelFolderSummary *summary,
+                                    CamelMessageInfo *info)
 {
 	CamelExchangeMessageInfo *einfo;
+	CamelFolderSummaryClass *folder_summary_class;
+
+	folder_summary_class = CAMEL_FOLDER_SUMMARY_CLASS (
+		camel_exchange_summary_parent_class);
 
 	einfo = (CamelExchangeMessageInfo *)info;
 
@@ -368,82 +237,146 @@ message_info_free (CamelFolderSummary *summary, CamelMessageInfo *info)
 	einfo->href = NULL;
 	einfo->thread_index = NULL;
 
-	CAMEL_FOLDER_SUMMARY_CLASS (parent_class)->message_info_free (summary, info);
+	folder_summary_class->message_info_free (summary, info);
+}
+
+static CamelFIRecord *
+exchange_summary_summary_header_to_db (CamelFolderSummary *s,
+                                       CamelException *ex)
+{
+	CamelExchangeSummary *exchange = (CamelExchangeSummary *) s;
+	CamelFolderSummaryClass *folder_summary_class;
+	struct _CamelFIRecord *fir;
+
+	folder_summary_class = CAMEL_FOLDER_SUMMARY_CLASS (
+		camel_exchange_summary_parent_class);
+
+	fir = folder_summary_class->summary_header_to_db (s, ex);
+	if (!fir)
+		return NULL;
+	fir->bdata = g_strdup_printf ("%u %u %u", exchange->version, exchange->readonly, exchange->high_article_num);
+
+	return fir;
+}
+
+static gint
+exchange_summary_summary_header_from_db (CamelFolderSummary *s,
+                                         CamelFIRecord *mir)
+{
+	CamelExchangeSummary *exchange = (CamelExchangeSummary *) s;
+	CamelFolderSummaryClass *folder_summary_class;
+	gchar *part;
+
+	folder_summary_class = CAMEL_FOLDER_SUMMARY_CLASS (
+		camel_exchange_summary_parent_class);
+
+	if (folder_summary_class->summary_header_from_db (s, mir) == -1)
+		return -1;
+
+	part = mir->bdata;
+
+	if (part) {
+		EXTRACT_FIRST_DIGIT (exchange->version)
+	}
+
+	if (part) {
+		EXTRACT_DIGIT (exchange->readonly)
+	}
+
+	if (part) {
+		EXTRACT_DIGIT (exchange->high_article_num)
+	}
+
+	return 0;
+}
+
+static CamelMIRecord *
+exchange_summary_message_info_to_db (CamelFolderSummary *s,
+                                     CamelMessageInfo *info)
+{
+	CamelExchangeMessageInfo *einfo = (CamelExchangeMessageInfo *)info;
+	CamelFolderSummaryClass *folder_summary_class;
+	struct _CamelMIRecord *mir;
+
+	folder_summary_class = CAMEL_FOLDER_SUMMARY_CLASS (
+		camel_exchange_summary_parent_class);
+
+	mir = folder_summary_class->message_info_to_db (s, info);
+	if (mir)
+		mir->bdata = g_strdup_printf ("%d-%s %d-%s", einfo->thread_index ? (gint)strlen(einfo->thread_index):0 , einfo->thread_index ? einfo->thread_index : "", einfo->href ? (gint)strlen(einfo->href):0, einfo->href ? einfo->href:"");
+
+	return mir;
+}
+
+static CamelMessageInfo *
+exchange_summary_message_info_from_db (CamelFolderSummary *s,
+                                       CamelMIRecord *mir)
+{
+	CamelMessageInfo *info;
+	CamelExchangeMessageInfo *einfo;
+	CamelFolderSummaryClass *folder_summary_class;
+
+	folder_summary_class = CAMEL_FOLDER_SUMMARY_CLASS (
+		camel_exchange_summary_parent_class);
+
+	info = folder_summary_class->message_info_from_db (s, mir);
+	if (info) {
+		gchar *part = g_strdup (mir->bdata);
+		gint len;
+		einfo = (CamelExchangeMessageInfo *)info;
+		EXTRACT_FIRST_STRING (einfo->thread_index)
+		EXTRACT_FIRST_STRING (einfo->href)
+	}
+
+	return info;
 }
 
 static gboolean
-check_for_trash (CamelFolder *folder)
+exchange_summary_info_set_flags (CamelMessageInfo *info,
+                                 guint32 flags,
+                                 guint32 set)
 {
-	CamelStore *store = (CamelStore *) folder->parent_store;
-	CamelException lex;
-	CamelFolder *trash;
-
-	camel_exception_init (&lex);
-	trash = camel_store_get_trash (store, &lex);
-
-	if (camel_exception_is_set (&lex) || !trash)
-		return FALSE;
-
-	return folder == trash;
-}
-
-static gboolean
-expunge_mail (CamelFolder *folder, CamelMessageInfo *info)
-{
-	CamelExchangeFolder *exchange_folder = (CamelExchangeFolder *) folder;
-	GPtrArray *uids = g_ptr_array_new ();
-	gchar *uid = g_strdup (info->uid);
-	CamelException lex;
-
-	g_ptr_array_add (uids, uid);
-
-	camel_exception_init (&lex);
-	camel_stub_send (exchange_folder->stub, &lex,
-			 CAMEL_STUB_CMD_EXPUNGE_UIDS,
-			 CAMEL_STUB_ARG_FOLDER, folder->full_name,
-			 CAMEL_STUB_ARG_STRINGARRAY, uids,
-			 CAMEL_STUB_ARG_END);
-
-	g_ptr_array_free (uids, TRUE);
-	return camel_exception_is_set (&lex);
-}
-
-static gboolean
-info_set_flags(CamelMessageInfo *info, guint32 flags, guint32 set)
-{
-	CamelFolder *folder = (CamelFolder *) info->summary->folder;
-	CamelOfflineStore *store = (CamelOfflineStore *) folder->parent_store;
+	CamelFolder *folder;
+	CamelStore *parent_store;
+	CamelOfflineStore *offline_store;
+	CamelFolderSummaryClass *folder_summary_class;
+	const gchar *full_name;
 
 	if (CAMEL_EXCHANGE_SUMMARY (info->summary)->readonly)
 		return FALSE;
 
-	if (store->state != CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
+	folder = info->summary->folder;
+	full_name = camel_folder_get_full_name (folder);
+	parent_store = camel_folder_get_parent_store (folder);
+
+	offline_store = CAMEL_OFFLINE_STORE (parent_store);
+
+	folder_summary_class = CAMEL_FOLDER_SUMMARY_CLASS (
+		camel_exchange_summary_parent_class);
+
+	if (offline_store->state != CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
 		if (folder && info->uid) {
 			if ((flags & set & CAMEL_MESSAGE_DELETED) &&
-			    check_for_trash (folder)) {
-				return expunge_mail (folder, info);
+			    exchange_summary_check_for_trash (folder)) {
+				return exchange_summary_expunge_mail (folder, info);
 			} else {
-				camel_stub_send_oneway (((CamelExchangeFolder *)folder)->stub,
-							CAMEL_STUB_CMD_SET_MESSAGE_FLAGS,
-							CAMEL_STUB_ARG_FOLDER, folder->full_name,
-							CAMEL_STUB_ARG_STRING, info->uid,
-							CAMEL_STUB_ARG_UINT32, set,
-							CAMEL_STUB_ARG_UINT32, flags,
-							CAMEL_STUB_ARG_END);
-				return CAMEL_FOLDER_SUMMARY_CLASS (parent_class)->info_set_flags(info, flags, set);
+				camel_exchange_utils_set_message_flags (
+					CAMEL_SERVICE (parent_store),
+					full_name, info->uid, set, flags, NULL);
+				return folder_summary_class->info_set_flags(info, flags, set);
 			}
 		}
 	}
 	else {
-		if(folder && info->uid) {
+		if (folder && info->uid) {
 			if ((flags & set & CAMEL_MESSAGE_DELETED) &&
-			    check_for_trash (folder)) {
+			    exchange_summary_check_for_trash (folder)) {
 				/* FIXME: should add a separate journal entry for this case. */ ;
 			} else {
 				CamelExchangeFolder *exchange_folder = (CamelExchangeFolder *) folder;
 				CamelExchangeJournal *journal = (CamelExchangeJournal *) exchange_folder->journal;
 				camel_exchange_journal_delete (journal, info->uid, flags, set, NULL);
-				return CAMEL_FOLDER_SUMMARY_CLASS (parent_class)->info_set_flags(info, flags, set);
+				return folder_summary_class->info_set_flags(info, flags, set);
 			}
 		}
 	}
@@ -451,27 +384,94 @@ info_set_flags(CamelMessageInfo *info, guint32 flags, guint32 set)
 }
 
 static gboolean
-info_set_user_tag(CamelMessageInfo *info, const gchar *name, const gchar *value)
+exchange_summary_info_set_user_tag (CamelMessageInfo *info,
+                                    const gchar *name,
+                                    const gchar *value)
 {
+	CamelFolderSummaryClass *folder_summary_class;
 	gint res;
 
 	if (CAMEL_EXCHANGE_SUMMARY (info->summary)->readonly)
 		return FALSE;
 
-	res = CAMEL_FOLDER_SUMMARY_CLASS (parent_class)->info_set_user_tag(info, name, value);
+	folder_summary_class = CAMEL_FOLDER_SUMMARY_CLASS (
+		camel_exchange_summary_parent_class);
+
+	res = folder_summary_class->info_set_user_tag(info, name, value);
 	if (res && info->summary->folder && info->uid) {
-		camel_stub_send_oneway (((CamelExchangeFolder *)info->summary->folder)->stub,
-					CAMEL_STUB_CMD_SET_MESSAGE_TAG,
-					CAMEL_STUB_ARG_FOLDER, info->summary->folder->full_name,
-					CAMEL_STUB_ARG_STRING, info->uid,
-					CAMEL_STUB_ARG_STRING, name,
-					CAMEL_STUB_ARG_STRING, value,
-					CAMEL_STUB_ARG_END);
+		CamelFolder *folder = info->summary->folder;
+		CamelStore *parent_store;
+		const gchar *full_name;
+
+		folder = info->summary->folder;
+		full_name = camel_folder_get_full_name (folder);
+		parent_store = camel_folder_get_parent_store (folder);
+
+		camel_exchange_utils_set_message_tag (
+			CAMEL_SERVICE (parent_store),
+			full_name, info->uid, name, value, NULL);
 	}
 
 	return res;
 }
 
+static void
+camel_exchange_summary_class_init (CamelExchangeSummaryClass *class)
+{
+	CamelFolderSummaryClass *folder_summary_class;
+
+	folder_summary_class = CAMEL_FOLDER_SUMMARY_CLASS (class);
+	folder_summary_class->message_info_size = sizeof (CamelExchangeMessageInfo);
+	folder_summary_class->content_info_size = sizeof (CamelMessageContentInfo);
+	folder_summary_class->summary_header_load = exchange_summary_header_load;
+	folder_summary_class->summary_header_save = exchange_summary_header_save;
+	folder_summary_class->message_info_migrate = exchange_summary_message_info_migrate;
+	folder_summary_class->message_info_new_from_header = exchange_summary_message_info_new_from_header;
+	folder_summary_class->message_info_free = exchange_summary_message_info_free;
+	folder_summary_class->summary_header_to_db = exchange_summary_summary_header_to_db;
+	folder_summary_class->summary_header_from_db = exchange_summary_summary_header_from_db;
+	folder_summary_class->message_info_to_db = exchange_summary_message_info_to_db;
+	folder_summary_class->message_info_from_db = exchange_summary_message_info_from_db;
+	folder_summary_class->info_set_flags = exchange_summary_info_set_flags;
+	folder_summary_class->info_set_user_tag = exchange_summary_info_set_user_tag;
+}
+
+static void
+camel_exchange_summary_init (CamelExchangeSummary *summary)
+{
+}
+
+/**
+ * camel_exchange_summary_new:
+ * @filename: filename to use for the summary
+ *
+ * Creates a new #CamelExchangeSummary based on @filename.
+ *
+ * Return value: the summary object.
+ **/
+CamelFolderSummary *
+camel_exchange_summary_new (struct _CamelFolder *folder, const gchar *filename)
+{
+	CamelFolderSummary *summary;
+	CamelException lex;
+
+	camel_exception_init (&lex);
+	summary = g_object_new (CAMEL_TYPE_EXCHANGE_SUMMARY, NULL);
+	summary->folder = folder;
+	camel_folder_summary_set_filename (summary, filename);
+	if (camel_folder_summary_load_from_db (summary, &lex) == -1) {
+		g_warning (
+			"Unable to load Exchage summary for folder %s: %s\n",
+			camel_folder_get_full_name (folder),
+			camel_exception_get_description(&lex));
+		camel_folder_summary_clear_db (summary);
+		camel_folder_summary_touch (summary);
+	}
+
+	camel_exception_clear (&lex);
+	return summary;
+}
+
 /**
  * camel_exchange_summary_get_readonly:
  * @summary: the summary
@@ -569,7 +569,7 @@ camel_exchange_summary_add_offline (CamelFolderSummary *summary,
 	const CamelTag *tag;
 
 	/* Create summary entry */
-	mi = (CamelMessageInfoBase *)camel_folder_summary_info_new_from_message (summary, message);
+	mi = (CamelMessageInfoBase *)camel_folder_summary_info_new_from_message (summary, message, NULL);
 
 	/* Copy flags 'n' tags */
 	mi->flags = camel_message_info_flags(info);
